"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var with_defaults_1 = require("../../utils/with-defaults");
var RawHoldable = /** @class */ (function (_super) {
    tslib_1.__extends(RawHoldable, _super);
    function RawHoldable() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onStartHold = function () {
            var _a = _this.props, interval = _a.interval, delay = _a.delay, onHold = _a.onHold;
            _this.clearTimers();
            _this._timeoutId = setTimeout(function () {
                _this._intervalId = setInterval(function () {
                    onHold && onHold();
                }, interval);
            }, delay);
        };
        _this.onEndHold = function () {
            _this.clearTimers();
        };
        return _this;
    }
    RawHoldable.prototype.componentWillUnmount = function () {
        this.clearTimers();
    };
    RawHoldable.prototype.componentDidUpdate = function (prevProps) {
        if (!prevProps.isDisabled && this.props.isDisabled) {
            this.clearTimers();
        }
    };
    RawHoldable.prototype.clearTimers = function () {
        clearTimeout(this._timeoutId);
        clearInterval(this._intervalId);
    };
    RawHoldable.prototype.render = function () {
        var _this = this;
        var children = this.props.children;
        var _a = children.props, onMouseDown = _a.onMouseDown, onMouseUp = _a.onMouseUp, onMouseLeave = _a.onMouseLeave, onTouchStart = _a.onTouchStart, onTouchEnd = _a.onTouchEnd, onTouchCancel = _a.onTouchCancel;
        return react_1.default.cloneElement(react_1.default.Children.only(children), {
            onMouseDown: function (e) {
                _this.onStartHold();
                onMouseDown && onMouseDown(e);
            },
            onMouseUp: function (e) {
                _this.onEndHold();
                onMouseUp && onMouseUp(e);
            },
            onMouseLeave: function (e) {
                _this.onEndHold();
                onMouseLeave && onMouseLeave(e);
            },
            onTouchStart: function (e) {
                _this.onStartHold();
                onTouchStart && onTouchStart(e);
            },
            onTouchEnd: function (e) {
                _this.onEndHold();
                onTouchEnd && onTouchEnd(e);
            },
            onTouchCancel: function (e) {
                _this.onEndHold();
                onTouchCancel && onTouchCancel(e);
            },
        });
    };
    return RawHoldable;
}(react_1.default.Component));
exports.Holdable = with_defaults_1.withDefaults({
    interval: 50,
    delay: 300,
})(RawHoldable);
//# sourceMappingURL=Holdable.js.map