"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PopoverPlacement;
(function (PopoverPlacement) {
    PopoverPlacement["Top"] = "Top";
    PopoverPlacement["Bottom"] = "Bottom";
    PopoverPlacement["Left"] = "Left";
    PopoverPlacement["Right"] = "Right";
})(PopoverPlacement = exports.PopoverPlacement || (exports.PopoverPlacement = {}));
var PopoverAlign;
(function (PopoverAlign) {
    PopoverAlign["Top"] = "Top";
    PopoverAlign["Bottom"] = "Bottom";
    PopoverAlign["Left"] = "Left";
    PopoverAlign["Right"] = "Right";
    PopoverAlign["Middle"] = "Middle";
    PopoverAlign["Center"] = "Center";
})(PopoverAlign = exports.PopoverAlign || (exports.PopoverAlign = {}));
function movePopoverVertically(_a) {
    var placement = _a.placement, align = _a.align, anchorTop = _a.anchorTop, anchorBottom = _a.anchorBottom, popoverHeight = _a.popoverHeight, parentHeight = _a.parentHeight, previousPlacement = _a.previousPlacement, _b = _a.checkBounds, checkBounds = _b === void 0 ? false : _b;
    switch (placement) {
        case PopoverPlacement.Top: {
            var top_1 = anchorTop - popoverHeight;
            if (checkBounds && top_1 < 0) {
                switch (previousPlacement) {
                    case PopoverPlacement.Bottom:
                    case PopoverPlacement.Top:
                        return movePopoverVertically({
                            placement: previousPlacement,
                            align: align,
                            anchorTop: anchorTop,
                            anchorBottom: anchorBottom,
                            popoverHeight: popoverHeight,
                            parentHeight: parentHeight,
                            previousPlacement: placement,
                        });
                    case PopoverPlacement.Left:
                    case PopoverPlacement.Right:
                        return undefined;
                }
                return movePopoverVertically({
                    placement: PopoverPlacement.Bottom,
                    align: align,
                    anchorTop: anchorTop,
                    anchorBottom: anchorBottom,
                    popoverHeight: popoverHeight,
                    parentHeight: parentHeight,
                    previousPlacement: placement,
                    checkBounds: true,
                });
            }
            return {
                top: top_1,
                placement: placement,
                align: align,
            };
        }
        case PopoverPlacement.Bottom: {
            var top_2 = anchorBottom;
            if (checkBounds && top_2 + popoverHeight > parentHeight) {
                switch (previousPlacement) {
                    case PopoverPlacement.Top:
                    case PopoverPlacement.Bottom:
                        return movePopoverVertically({
                            placement: previousPlacement,
                            align: align,
                            anchorTop: anchorTop,
                            anchorBottom: anchorBottom,
                            popoverHeight: popoverHeight,
                            parentHeight: parentHeight,
                            previousPlacement: placement,
                        });
                    case PopoverPlacement.Left:
                    case PopoverPlacement.Right:
                        return undefined;
                }
                return movePopoverVertically({
                    placement: PopoverPlacement.Top,
                    align: align,
                    anchorTop: anchorTop,
                    anchorBottom: anchorBottom,
                    popoverHeight: popoverHeight,
                    parentHeight: parentHeight,
                    previousPlacement: placement,
                    checkBounds: true,
                });
            }
            return {
                top: top_2,
                placement: placement,
                align: align,
            };
        }
    }
    switch (align) {
        case PopoverAlign.Top: {
            var top_3 = anchorTop;
            if (checkBounds && top_3 + popoverHeight > parentHeight) {
                var resultForMiddle = movePopoverVertically({
                    placement: placement,
                    align: PopoverAlign.Middle,
                    anchorTop: anchorTop,
                    anchorBottom: anchorBottom,
                    popoverHeight: popoverHeight,
                    parentHeight: parentHeight,
                });
                if (resultForMiddle && resultForMiddle.top + popoverHeight > parentHeight) {
                    return movePopoverVertically({
                        placement: placement,
                        align: PopoverAlign.Bottom,
                        anchorTop: anchorTop,
                        anchorBottom: anchorBottom,
                        popoverHeight: popoverHeight,
                        parentHeight: parentHeight,
                    });
                }
                return resultForMiddle;
            }
            return {
                top: top_3,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Middle: {
            var top_4 = anchorTop + (anchorBottom - anchorTop) / 2 - popoverHeight / 2;
            if (checkBounds) {
                if (top_4 < 0) {
                    return movePopoverVertically({
                        placement: placement,
                        align: PopoverAlign.Top,
                        anchorTop: anchorTop,
                        anchorBottom: anchorBottom,
                        popoverHeight: popoverHeight,
                        parentHeight: parentHeight,
                    });
                }
                else if (top_4 + popoverHeight > parentHeight) {
                    return movePopoverVertically({
                        placement: placement,
                        align: PopoverAlign.Bottom,
                        anchorTop: anchorTop,
                        anchorBottom: anchorBottom,
                        popoverHeight: popoverHeight,
                        parentHeight: parentHeight,
                    });
                }
            }
            return {
                top: top_4,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Bottom: {
            var top_5 = anchorBottom - popoverHeight;
            if (checkBounds && top_5 < 0) {
                var resultForMiddle = movePopoverVertically({
                    placement: placement,
                    align: PopoverAlign.Middle,
                    anchorTop: anchorTop,
                    anchorBottom: anchorBottom,
                    popoverHeight: popoverHeight,
                    parentHeight: parentHeight,
                });
                if (resultForMiddle && resultForMiddle.top < 0) {
                    return movePopoverVertically({
                        placement: placement,
                        align: PopoverAlign.Top,
                        anchorTop: anchorTop,
                        anchorBottom: anchorBottom,
                        popoverHeight: popoverHeight,
                        parentHeight: parentHeight,
                    });
                }
                return resultForMiddle;
            }
            return {
                top: top_5,
                placement: placement,
                align: align,
            };
        }
    }
    return undefined;
}
exports.movePopoverVertically = movePopoverVertically;
function movePopoverHorizontally(_a) {
    var placement = _a.placement, align = _a.align, anchorLeft = _a.anchorLeft, anchorRight = _a.anchorRight, popoverWidth = _a.popoverWidth, parentWidth = _a.parentWidth, _b = _a.checkBounds, checkBounds = _b === void 0 ? false : _b, previousPlacement = _a.previousPlacement;
    switch (placement) {
        case PopoverPlacement.Left: {
            var left = anchorLeft - popoverWidth;
            if (checkBounds && left < 0) {
                switch (previousPlacement) {
                    case PopoverPlacement.Right:
                    case PopoverPlacement.Left:
                        return movePopoverHorizontally({
                            placement: previousPlacement,
                            align: align,
                            anchorLeft: anchorLeft,
                            anchorRight: anchorRight,
                            popoverWidth: popoverWidth,
                            parentWidth: parentWidth,
                            previousPlacement: placement,
                        });
                    case PopoverPlacement.Top:
                    case PopoverPlacement.Bottom:
                        return undefined;
                }
                return movePopoverHorizontally({
                    placement: PopoverPlacement.Right,
                    align: align,
                    anchorLeft: anchorLeft,
                    anchorRight: anchorRight,
                    popoverWidth: popoverWidth,
                    parentWidth: parentWidth,
                    previousPlacement: placement,
                    checkBounds: true,
                });
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverPlacement.Right: {
            var left = anchorRight;
            if (checkBounds && left + popoverWidth > parentWidth) {
                switch (previousPlacement) {
                    case PopoverPlacement.Left:
                    case PopoverPlacement.Right:
                        return movePopoverHorizontally({
                            placement: previousPlacement,
                            align: align,
                            anchorLeft: anchorLeft,
                            anchorRight: anchorRight,
                            popoverWidth: popoverWidth,
                            parentWidth: parentWidth,
                            previousPlacement: placement,
                        });
                    case PopoverPlacement.Top:
                    case PopoverPlacement.Bottom:
                        return undefined;
                }
                return movePopoverHorizontally({
                    placement: PopoverPlacement.Left,
                    align: align,
                    anchorLeft: anchorLeft,
                    anchorRight: anchorRight,
                    popoverWidth: popoverWidth,
                    parentWidth: parentWidth,
                    previousPlacement: placement,
                    checkBounds: true,
                });
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
    }
    switch (align) {
        case PopoverAlign.Left: {
            var left = anchorLeft;
            if (checkBounds && left + popoverWidth > parentWidth) {
                var resultForCenter = movePopoverHorizontally({
                    placement: placement,
                    align: PopoverAlign.Center,
                    anchorLeft: anchorLeft,
                    anchorRight: anchorRight,
                    popoverWidth: popoverWidth,
                    parentWidth: parentWidth,
                });
                if (resultForCenter && resultForCenter.left + popoverWidth > parentWidth) {
                    return movePopoverHorizontally({
                        placement: placement,
                        align: PopoverAlign.Right,
                        anchorLeft: anchorLeft,
                        anchorRight: anchorRight,
                        popoverWidth: popoverWidth,
                        parentWidth: parentWidth,
                    });
                }
                return resultForCenter;
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Center: {
            var left = anchorLeft + (anchorRight - anchorLeft) / 2 - popoverWidth / 2;
            if (checkBounds) {
                if (left < 0) {
                    return movePopoverHorizontally({
                        placement: placement,
                        align: PopoverAlign.Left,
                        anchorLeft: anchorLeft,
                        anchorRight: anchorRight,
                        popoverWidth: popoverWidth,
                        parentWidth: parentWidth,
                    });
                }
                else if (left + popoverWidth > parentWidth) {
                    return movePopoverHorizontally({
                        placement: placement,
                        align: PopoverAlign.Right,
                        anchorLeft: anchorLeft,
                        anchorRight: anchorRight,
                        popoverWidth: popoverWidth,
                        parentWidth: parentWidth,
                    });
                }
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Right: {
            var left = anchorRight - popoverWidth;
            if (checkBounds && left < 0) {
                var resultForCenter = movePopoverHorizontally({
                    placement: placement,
                    align: PopoverAlign.Center,
                    anchorLeft: anchorLeft,
                    anchorRight: anchorRight,
                    popoverWidth: popoverWidth,
                    parentWidth: parentWidth,
                });
                if (resultForCenter && resultForCenter.left < 0) {
                    return movePopoverHorizontally({
                        placement: placement,
                        align: PopoverAlign.Left,
                        anchorLeft: anchorLeft,
                        anchorRight: anchorRight,
                        popoverWidth: popoverWidth,
                        parentWidth: parentWidth,
                    });
                }
                return resultForCenter;
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
    }
    return undefined;
}
exports.movePopoverHorizontally = movePopoverHorizontally;
function getPlacementModifier(placement) {
    switch (placement) {
        case PopoverPlacement.Bottom: {
            return 'placementBottom';
        }
        case PopoverPlacement.Top: {
            return 'placementTop';
        }
        case PopoverPlacement.Left: {
            return 'placementLeft';
        }
        case PopoverPlacement.Right: {
            return 'placementRight';
        }
    }
}
exports.getPlacementModifier = getPlacementModifier;
//# sourceMappingURL=Popover.model.js.map