"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var classnames_1 = tslib_1.__importDefault(require("classnames"));
var Scrollable_const_1 = require("./Scrollable.const");
var Scrollbar_1 = require("../Scrollbar/Scrollbar");
var ResizeDetector_1 = require("../ResizeDetector/ResizeDetector");
var HorizontalScrollbar_1 = require("../Scrollbar/HorizontalScrollbar");
var VerticalScrollbar_1 = require("../Scrollbar/VerticalScrollbar");
var Scrollbar_util_1 = tslib_1.__importDefault(require("../Scrollbar/Scrollbar.util"));
var withTheme_1 = require("../../utils/withTheme");
var with_defaults_1 = require("../../utils/with-defaults");
var react_2 = require("react");
exports.SCROLLABLE = Symbol('Scrollable');
var RawScrollable = /** @class */ (function (_super) {
    tslib_1.__extends(RawScrollable, _super);
    function RawScrollable() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._withHorizontalScrollbar = false;
        _this._withVerticalScrollbar = false;
        _this.state = {
            container: void 0,
            scrollable: void 0,
        };
        _this.onScroll = function (event) {
            var _a = event.target, scrollLeft = _a.scrollLeft, scrollTop = _a.scrollTop;
            var onScroll = _this.props.onScroll;
            onScroll && onScroll(scrollLeft, scrollTop);
        };
        _this.onScrollbarUpdate = function (type, isVisible) {
            var onUpdate = _this.props.onUpdate;
            switch (type) {
                case Scrollbar_1.SCROLLBAR_TYPE.VERTICAL: {
                    _this._withVerticalScrollbar = isVisible;
                    break;
                }
                case Scrollbar_1.SCROLLBAR_TYPE.HORIZONTAL: {
                    _this._withHorizontalScrollbar = isVisible;
                    break;
                }
            }
            _this.forceUpdate();
            onUpdate && onUpdate(_this._withHorizontalScrollbar, _this._withVerticalScrollbar);
        };
        _this.onResize = function () {
            _this._emitter.emit(Scrollable_const_1.EVENT_SCROLABLE.RESIZE);
        };
        return _this;
    }
    RawScrollable.prototype.getChildContext = function () {
        var _a;
        var theme = this.props.theme;
        return _a = {
                size: Scrollbar_util_1.default(theme.container)
            },
            _a[Scrollable_const_1.SCROLLABLE_CONTEXT_EMITTER.toString()] = this._emitter,
            _a;
    };
    RawScrollable.prototype.componentDidMount = function () {
        this.setState({
            container: this._container,
        });
        this._emitter.on(Scrollable_const_1.EVENT_SCROLABLE.SCROLLBAR_UPDATE, this.onScrollbarUpdate);
        this._emitter.on(Scrollable_const_1.EVENT_SCROLABLE.SCROLL, this.onScroll);
    };
    RawScrollable.prototype.componentWillUnmount = function () {
        this._emitter.off(Scrollable_const_1.EVENT_SCROLABLE.SCROLLBAR_UPDATE, this.onScrollbarUpdate);
        this._emitter.off(Scrollable_const_1.EVENT_SCROLABLE.SCROLL, this.onScroll);
    };
    RawScrollable.prototype.componentWillMount = function () {
        this._emitter = new Scrollable_const_1.ScrollableInternalEmitter();
    };
    RawScrollable.prototype.render = function () {
        var _a;
        var _this = this;
        var _b = this.props, theme = _b.theme, ResizeDetector = _b.ResizeDetector, VerticalScrollbar = _b.VerticalScrollbar, HorizontalScrollbar = _b.HorizontalScrollbar, shouldOverlayContent = _b.shouldOverlayContent;
        var children = react_2.Children.only(this.props.children);
        var container = this.state.container;
        var className = classnames_1.default(theme.scrollable, (_a = {},
            _a[theme.withHorizontalScrollbar] = this._withHorizontalScrollbar,
            _a[theme.withVerticalScrollbar] = this._withVerticalScrollbar,
            _a[theme.overlayContent] = shouldOverlayContent,
            _a), (children && children.props && children.props.className) || '');
        var containerClassName = classnames_1.default(theme.scrollbar, {
            withBothScrollabars: this._withHorizontalScrollbar && this._withVerticalScrollbar,
        });
        var resizeDetectorProps = {
            theme: {
                container: theme.resizeDetector,
            },
            onResize: this.onResize,
        };
        return (react_1.default.createElement("div", { className: className, ref: function (el) { return (_this._scrollable = el); } },
            react_1.default.createElement("div", { className: theme.wrapper },
                react_1.default.createElement("div", { className: theme.container, ref: function (el) { return (_this._container = el); } },
                    react_1.default.createElement("div", { className: theme.content },
                        react_2.cloneElement(children, {
                            className: null,
                        }),
                        react_1.default.createElement(ResizeDetector, tslib_1.__assign({}, resizeDetectorProps)))),
                container && [
                    react_1.default.createElement(HorizontalScrollbar, { ref: function (el) { return (_this._horizontalScrollbar = el); }, key: "horizontalScrollbar", container: container, scrollLeft: this.props.scrollLeft, theme: {
                            bar: theme.horizontal_scrollbar__bar,
                            container: containerClassName,
                        } }),
                    react_1.default.createElement(VerticalScrollbar, { ref: function (el) { return (_this._verticalScrollbar = el); }, key: "verticalScrollbar", container: container, scrollTop: this.props.scrollTop, theme: {
                            bar: theme.vertical_scrollbar__bar,
                            container: containerClassName,
                        } }),
                ]),
            react_1.default.createElement(ResizeDetector, tslib_1.__assign({}, resizeDetectorProps))));
    };
    RawScrollable.childContextTypes = Scrollable_const_1.CONTEXT_TYPES;
    return RawScrollable;
}(react_2.Component));
exports.RawScrollable = RawScrollable;
exports.Scrollable = withTheme_1.withTheme(exports.SCROLLABLE)(with_defaults_1.withDefaults({
    ResizeDetector: ResizeDetector_1.ResizeDetector,
    VerticalScrollbar: VerticalScrollbar_1.VerticalScrollbar,
    HorizontalScrollbar: HorizontalScrollbar_1.HorizontalScrollbar,
})(RawScrollable));
//# sourceMappingURL=Scrollable.js.map