"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var react_dom_1 = tslib_1.__importDefault(require("react-dom"));
var classnames_1 = tslib_1.__importDefault(require("classnames"));
var Bar_1 = require("./Bar");
var Scrollable_const_1 = require("../Scrollable/Scrollable.const");
exports.SCROLLBAR = Symbol('Scrollbar');
var SCROLLBAR_TYPE;
(function (SCROLLBAR_TYPE) {
    SCROLLBAR_TYPE["HORIZONTAL"] = "HORIZONTAL";
    SCROLLBAR_TYPE["VERTICAL"] = "VERTICAL";
})(SCROLLBAR_TYPE = exports.SCROLLBAR_TYPE || (exports.SCROLLBAR_TYPE = {}));
var Scrollbar = /** @class */ (function (_super) {
    tslib_1.__extends(Scrollbar, _super);
    function Scrollbar(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.state = {
            isVisible: false,
        };
        /**
         * @param {Event} event
         * @private
         */
        _this._onContainerScroll = function (event) {
            _this._updateBar();
            var emitter = _this.context[Scrollable_const_1.SCROLLABLE_CONTEXT_EMITTER];
            emitter.emit(Scrollable_const_1.EVENT_SCROLABLE.SCROLL, event);
        };
        _this.onResize = function () {
            _this._update();
        };
        ////////////////////////
        // DOM EVENT HANDLERS //
        ////////////////////////
        /**
         * @abstract
         * @param {WheelEvent} event
         * @protected
         */
        _this.onTrackMouseWheel = function (event) { };
        /**
         * @abstract
         * @param {MouseEvent} event
         * @protected
         */
        _this.onTrackClick = function (event) { };
        /**
         * @abstract
         * @param {MouseEvent} event
         * @protected
         */
        _this.onBarDragStart = function (event) { };
        /**
         * @abstract
         * @param {MouseEvent} event
         * @protected
         */
        _this.onBarDrag = function (event) { };
        var container = _this.props.container;
        _this._container = container;
        return _this;
    }
    Scrollbar.prototype.componentDidMount = function () {
        var emitter = this.context[Scrollable_const_1.SCROLLABLE_CONTEXT_EMITTER];
        emitter.on(Scrollable_const_1.EVENT_SCROLABLE.RESIZE, this.onResize);
        if (this._container) {
            this._container.addEventListener('scroll', this._onContainerScroll);
        }
    };
    Scrollbar.prototype.componentWillUnmount = function () {
        var emitter = this.context[Scrollable_const_1.SCROLLABLE_CONTEXT_EMITTER];
        emitter.off(Scrollable_const_1.EVENT_SCROLABLE.RESIZE, this.onResize);
        if (this._container) {
            this._container.removeEventListener('scroll', this._onContainerScroll);
        }
    };
    Scrollbar.prototype.render = function () {
        var _a;
        var _this = this;
        var theme = this.props.theme;
        var isVisible = this.state.isVisible;
        var className = classnames_1.default(theme.container, (_a = {},
            _a[theme.containerIsVisible] = isVisible,
            _a));
        var barProps = {
            theme: {
                container: theme.bar,
            },
            onBarDragStart: this.onBarDragStart,
            onBarDrag: this.onBarDrag,
        };
        return (react_1.default.createElement("div", { className: className, ref: function (el) { return (_this._scrollbar = el); } },
            react_1.default.createElement("div", { className: theme.track, onWheel: this.onTrackMouseWheel, onClick: this.onTrackClick, ref: function (el) { return (_this._track = el); } },
                react_1.default.createElement(Bar_1.Bar, tslib_1.__assign({}, barProps, { ref: function (el) { return (_this._bar = react_dom_1.default.findDOMNode(el)); } })))));
    };
    Scrollbar.prototype._updateBar = function () { };
    Scrollbar.prototype._update = function () { };
    Scrollbar.contextTypes = Scrollable_const_1.CONTEXT_TYPES;
    return Scrollbar;
}(react_1.default.Component));
exports.Scrollbar = Scrollbar;
//# sourceMappingURL=Scrollbar.js.map