"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Scrollbar_1 = require("./Scrollbar");
var Scrollable_const_1 = require("../Scrollable/Scrollable.const");
var pure_1 = require("../../utils/pure");
var withTheme_1 = require("../../utils/withTheme");
exports.VERTICAL_SCROLLBAR = Symbol('VerticalScrollbar');
var RawVerticalScrollbar = /** @class */ (function (_super) {
    tslib_1.__extends(RawVerticalScrollbar, _super);
    function RawVerticalScrollbar() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._scrollTo = function (position) {
            if (_this._container) {
                _this._container.scrollTop = position;
            }
        };
        ////////////////////////
        // DOM EVENT HANDLERS //
        ////////////////////////
        /**
         * @param {WheelEvent} event
         * @protected
         */
        _this.onTrackMouseWheel = function (event) {
            if (_this._container) {
                _this._container.scrollTop += event.deltaY * 10 || event['detail'] * 10 || event['wheelDelta'] * -1;
            }
        };
        /**
         * @param {MouseEvent} event
         * @protected
         */
        _this.onTrackClick = function (event) {
            if (_this._track && _this._container) {
                var trackPosition = _this._track.getBoundingClientRect().top + window.pageYOffset - document.documentElement.clientTop;
                _this._container.scrollTop = Math.floor((event.clientY - trackPosition) / _this._ratio.position);
            }
        };
        /**
         * @param {MouseEvent} event
         * @protected
         */
        _this.onBarDragStart = function (event) {
            _this._previousDragCoordinate = event.clientY;
        };
        /**
         * @param {MouseEvent} event
         * @protected
         */
        _this.onBarDrag = function (event) {
            if (_this._container) {
                var delta = _this._previousDragCoordinate - event.clientY;
                _this._previousDragCoordinate = event.clientY;
                _this._container.scrollTop -= delta / _this._ratio.position;
            }
        };
        return _this;
    }
    /**
     * @returns {Number}
     * @protected
     */
    RawVerticalScrollbar.prototype._getMinBarSize = function () {
        return parseFloat(window.getComputedStyle(this._bar).getPropertyValue('min-height'));
    };
    /**
     * Hide native scroll
     * @private
     */
    RawVerticalScrollbar.prototype._hideNativeScrollContainer = function () {
        var width = this.context.size.width;
        if (this._container && width) {
            this._container.style.marginRight = "-" + width + "px";
            this._container.style.width = "calc(100% + " + width + "px)";
        }
    };
    RawVerticalScrollbar.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        this._minBarSize = this._getMinBarSize();
        this._hideNativeScrollContainer();
        this._update();
        if (this.props.scrollTop) {
            this._scrollTo(this.props.scrollTop);
        }
    };
    RawVerticalScrollbar.prototype.componentWillReceiveProps = function (newProps) {
        if (this.props.scrollTop !== newProps.scrollTop && newProps.scrollTop) {
            this._scrollTo(newProps.scrollTop);
        }
    };
    RawVerticalScrollbar.prototype._update = function () {
        this._toggle();
        this._ratio = this._getRatio();
        this._updateBar();
    };
    RawVerticalScrollbar.prototype._toggle = function () {
        if (this._container) {
            var bounds = this._container.getBoundingClientRect();
            var height = Math.round(bounds.height);
            var scrollHeight = this._container.scrollHeight;
            var isVisible = scrollHeight > height;
            if (isVisible !== this.state.isVisible) {
                this.setState({
                    isVisible: isVisible,
                });
                var emitter = this.context[Scrollable_const_1.SCROLLABLE_CONTEXT_EMITTER];
                emitter.emit(Scrollable_const_1.EVENT_SCROLABLE.SCROLLBAR_UPDATE, Scrollbar_1.SCROLLBAR_TYPE.VERTICAL, isVisible);
            }
        }
    };
    RawVerticalScrollbar.prototype._updateBar = function () {
        if (this._container) {
            var visibleHeight = this._container.clientHeight;
            var barHeight = Math.ceil(visibleHeight * this._ratio.size);
            this._bar.style.height = barHeight + "px";
            //position
            var scrollTop = this._container.scrollTop;
            var barPositionTop = Math.floor(scrollTop * this._ratio.position);
            this._bar.style.top = barPositionTop + "px";
        }
    };
    RawVerticalScrollbar.prototype._getRatio = function () {
        if (this._container && this._track) {
            var _a = this._container, scrollSize = _a.scrollHeight, containerSize = _a.clientHeight;
            var trackSize = this._track.offsetHeight;
            var sizeRatio = trackSize / scrollSize;
            var possibleBarSize = containerSize * sizeRatio;
            var barSize = possibleBarSize < this._minBarSize ? this._minBarSize : possibleBarSize;
            var hiddenContentAreaSize = scrollSize - containerSize;
            var hiddenTrackAreaSize = trackSize - barSize;
            var posRatio = hiddenTrackAreaSize / hiddenContentAreaSize;
            return {
                size: sizeRatio,
                position: posRatio,
            };
        }
        else {
            return {
                size: 0,
                position: 0,
            };
        }
    };
    RawVerticalScrollbar = tslib_1.__decorate([
        pure_1.PURE
    ], RawVerticalScrollbar);
    return RawVerticalScrollbar;
}(Scrollbar_1.Scrollbar));
exports.RawVerticalScrollbar = RawVerticalScrollbar;
exports.VerticalScrollbar = withTheme_1.withTheme(exports.VERTICAL_SCROLLBAR)(RawVerticalScrollbar);
//# sourceMappingURL=VerticalScrollbar.js.map