"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var react_2 = require("react");
var pure_1 = require("../../utils/pure");
var Control_1 = require("../Control/Control");
var classnames_1 = tslib_1.__importDefault(require("classnames"));
var withTheme_1 = require("../../utils/withTheme");
var SteppableInput_1 = require("../SteppableInput/SteppableInput");
var with_defaults_1 = require("../../utils/with-defaults");
var Option_1 = require("fp-ts/lib/Option");
var TimeInput_model_1 = require("./TimeInput.model");
var pipeable_1 = require("fp-ts/lib/pipeable");
exports.TIME_INPUT = Symbol('TimeInput');
var RawTimeInput = /** @class */ (function (_super) {
    tslib_1.__extends(RawTimeInput, _super);
    function RawTimeInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        // Helper, which helps to detect order of input in the two-digit field;
        _this.secondInput = false;
        _this.getSectionClassName = function (section) {
            var _a;
            var _b = _this.props, theme = _b.theme, isDisabled = _b.isDisabled;
            var activeSection = _this.state.activeSection;
            return classnames_1.default(theme.section, (_a = {},
                _a[theme.section_isActive] = !isDisabled && activeSection === section,
                _a));
        };
        _this.onHoursMouseDown = function () {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: TimeInput_model_1.Section.Hours,
                });
                _this.secondInput = false;
                _this.correctTimeAndUpdate();
            }
        };
        _this.onMinutesMouseDown = function () {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: TimeInput_model_1.Section.Minutes,
                });
                _this.secondInput = false;
                _this.correctTimeAndUpdate();
            }
        };
        _this.onSecondsMouseDown = function () {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: TimeInput_model_1.Section.Seconds,
                });
                _this.secondInput = false;
                _this.correctTimeAndUpdate();
            }
        };
        _this.onPeriodTypeMouseDown = function () {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: TimeInput_model_1.Section.PeriodType,
                });
                _this.secondInput = false;
                _this.correctTimeAndUpdate();
            }
        };
        _this.onIncrement = function () {
            _this.secondInput = false;
            _this.step(1);
        };
        _this.onDecrement = function () {
            _this.secondInput = false;
            _this.step(-1);
        };
        _this.onClear = function () {
            _this.secondInput = false;
            _this.updateTime(Option_1.none, Option_1.none, Option_1.none, Option_1.none);
        };
        _this.onFocus = function () {
            _this.secondInput = false;
            if (!TimeInput_model_1.isDefined(_this.state.activeSection)) {
                _this.setState({
                    activeSection: TimeInput_model_1.Section.Hours,
                });
            }
        };
        _this.onBlur = function () {
            _this.secondInput = false;
            _this.correctTimeAndUpdate();
            _this.setState({
                activeSection: undefined,
            });
        };
        _this.onKeyDown = function (e) {
            var activeSection = _this.state.activeSection;
            var _a = _this.props, withSeconds = _a.withSeconds, withPeriodType = _a.withPeriodType;
            var _b = _this.props.value, hours = _b.hours, minutes = _b.minutes, seconds = _b.seconds, periodType = _b.periodType;
            switch (e.keyCode) {
                case Control_1.KeyCode.Left: {
                    e.preventDefault(); //block h-scrolling
                    _this.secondInput = false;
                    _this.correctTimeAndUpdate();
                    _this.setState({
                        activeSection: TimeInput_model_1.findActiveSectionOnKeyLeft(activeSection, withSeconds),
                    });
                    break;
                }
                case Control_1.KeyCode.Right: {
                    e.preventDefault(); //block h-scrolling
                    _this.secondInput = false;
                    _this.correctTimeAndUpdate();
                    _this.setState({
                        activeSection: TimeInput_model_1.findActiveSectionOnKeyRight(activeSection, withSeconds, withPeriodType),
                    });
                    break;
                }
                case Control_1.KeyCode.Delete: //fallthrough
                case Control_1.KeyCode.Backspace: {
                    _this.secondInput = false;
                    switch (activeSection) {
                        case TimeInput_model_1.Section.Hours: {
                            _this.updateTime(Option_1.none, minutes, seconds, periodType);
                            break;
                        }
                        case TimeInput_model_1.Section.Minutes: {
                            _this.updateTime(hours, Option_1.none, seconds, periodType);
                            break;
                        }
                        case TimeInput_model_1.Section.Seconds: {
                            _this.updateTime(hours, minutes, Option_1.none, periodType);
                            break;
                        }
                    }
                    break;
                }
                default: {
                    var keyCode = e.keyCode;
                    var number = Control_1.KEY_CODE_NUM_MAP[keyCode];
                    var letter = Control_1.KEY_CODE_LETR_MAP[keyCode];
                    if (TimeInput_model_1.isDefined(number)) {
                        _this.handleDigitKeyDown(number);
                    }
                    else if (TimeInput_model_1.isDefined(letter)) {
                        _this.handleLetterKeyDown(letter);
                    }
                }
            }
        };
        return _this;
    }
    RawTimeInput.prototype.render = function () {
        var _a = this.props, theme = _a.theme, decrementIcon = _a.decrementIcon, incrementIcon = _a.incrementIcon, isDisabled = _a.isDisabled, clearIcon = _a.clearIcon, error = _a.error, value = _a.value, SteppableInput = _a.SteppableInput;
        var hours = value.hours, minutes = value.minutes, seconds = value.seconds, periodType = value.periodType;
        var _b = this.props, withSeconds = _b.withSeconds, withPeriodType = _b.withPeriodType;
        var onClear = Option_1.isSome(hours) || Option_1.isSome(minutes) ? this.onClear : undefined;
        return (react_1.default.createElement(SteppableInput, { isDisabled: isDisabled, theme: theme.SteppableInput, onBlur: this.onBlur, error: error, onFocus: this.onFocus, decrementIcon: decrementIcon, incrementIcon: incrementIcon, clearIcon: clearIcon, onKeyDown: this.onKeyDown, onClear: onClear, onDecrement: this.onDecrement, onIncrement: this.onIncrement },
            react_1.default.createElement("div", { className: theme.inner },
                react_1.default.createElement("span", { className: this.getSectionClassName(TimeInput_model_1.Section.Hours), onMouseDown: this.onHoursMouseDown }, TimeInput_model_1.renderSection(pipeable_1.pipe(hours, Option_1.map(TimeInput_model_1.formatNumericValue)))),
                react_1.default.createElement("span", { className: theme.separator }, ":"),
                react_1.default.createElement("span", { className: this.getSectionClassName(TimeInput_model_1.Section.Minutes), onMouseDown: this.onMinutesMouseDown }, TimeInput_model_1.renderSection(pipeable_1.pipe(minutes, Option_1.map(TimeInput_model_1.formatNumericValue)))),
                withSeconds && (react_1.default.createElement(react_2.Fragment, null,
                    react_1.default.createElement("span", { className: theme.separator }, ":"),
                    react_1.default.createElement("span", { className: this.getSectionClassName(TimeInput_model_1.Section.Seconds), onMouseDown: this.onSecondsMouseDown }, TimeInput_model_1.renderSection(pipeable_1.pipe(seconds, Option_1.map(TimeInput_model_1.formatNumericValue)))))),
                withPeriodType && (react_1.default.createElement(react_2.Fragment, null,
                    "\u00A0",
                    react_1.default.createElement("span", { className: this.getSectionClassName(TimeInput_model_1.Section.PeriodType), onMouseDown: this.onPeriodTypeMouseDown }, TimeInput_model_1.renderSection(pipeable_1.pipe(periodType, Option_1.map(TimeInput_model_1.formatTimePeriod)))))))));
    };
    RawTimeInput.prototype.handleLetterKeyDown = function (letter) {
        var _a = this.props.value, hours = _a.hours, minutes = _a.minutes, seconds = _a.seconds;
        var activeSection = this.state.activeSection;
        if (activeSection === TimeInput_model_1.Section.PeriodType) {
            if (letter === Control_1.KEY_CODE_LETR_MAP[Control_1.KeyCode.LETR_A]) {
                this.updateTime(hours, minutes, seconds, Option_1.some(TimeInput_model_1.PeriodType.AM));
            }
            else if (letter === Control_1.KEY_CODE_LETR_MAP[Control_1.KeyCode.LETR_P]) {
                this.updateTime(hours, minutes, seconds, Option_1.some(TimeInput_model_1.PeriodType.PM));
            }
        }
    };
    RawTimeInput.prototype.handleDigitKeyDown = function (digit) {
        var activeSection = this.state.activeSection;
        var _a = this.props, _b = _a.withPeriodType, withPeriodType = _b === void 0 ? false : _b, _c = _a.withSeconds, withSeconds = _c === void 0 ? false : _c;
        var _d = this.props.value, hours = _d.hours, minutes = _d.minutes, seconds = _d.seconds, periodType = _d.periodType;
        switch (activeSection) {
            case TimeInput_model_1.Section.Hours: {
                if (this.secondInput && Option_1.isSome(hours)) {
                    var newHours = void 0;
                    if (!withPeriodType) {
                        if (hours.value < 2) {
                            newHours = Number("" + hours.value + digit);
                        }
                        else if (hours.value === 2) {
                            newHours = Math.min(Number("" + hours.value + digit), TimeInput_model_1.MAX_VALID_HOURS_FOR_24H_FORMAT);
                        }
                        else {
                            newHours = digit;
                        }
                        this.updateTime(Option_1.some(newHours), minutes, seconds, periodType);
                        this.setState({
                            activeSection: TimeInput_model_1.Section.Minutes,
                        });
                        this.secondInput = false;
                    }
                    else {
                        if (hours.value < 2) {
                            newHours = Math.min(Number("" + hours.value + digit), TimeInput_model_1.MAX_VALID_HOURS_FOR_12H_FORMAT);
                        }
                        else {
                            newHours = digit;
                        }
                        this.updateTime(Option_1.some(newHours), minutes, seconds, periodType);
                        this.setState({
                            activeSection: TimeInput_model_1.Section.Minutes,
                        });
                        this.secondInput = false;
                    }
                }
                else {
                    this.updateTime(Option_1.some(digit), minutes, seconds, periodType);
                    if (digit > 2 && !withPeriodType) {
                        this.setState({
                            activeSection: TimeInput_model_1.Section.Minutes,
                        });
                        this.secondInput = false;
                    }
                    else if (digit > 1 && withPeriodType) {
                        this.setState({
                            activeSection: TimeInput_model_1.Section.Minutes,
                        });
                        this.secondInput = false;
                    }
                    else {
                        this.secondInput = true;
                    }
                }
                break;
            }
            case TimeInput_model_1.Section.Minutes: {
                if (!withSeconds) {
                    var newMinutes = void 0;
                    if (this.secondInput) {
                        newMinutes = Option_1.isSome(minutes) ? Number("" + minutes.value % 10 + digit) : digit;
                    }
                    else {
                        newMinutes = digit;
                        this.secondInput = true;
                    }
                    this.updateTime(hours, Option_1.some(newMinutes), seconds, periodType);
                }
                else {
                    if (this.secondInput && Option_1.isSome(minutes)) {
                        var newMinutes = Number("" + minutes.value + digit);
                        this.updateTime(hours, Option_1.some(newMinutes), seconds, periodType);
                        if (withSeconds) {
                            this.setState({
                                activeSection: TimeInput_model_1.Section.Seconds,
                            });
                        }
                        else if (withPeriodType) {
                            this.setState({
                                activeSection: TimeInput_model_1.Section.PeriodType,
                            });
                        }
                        this.updateTime(hours, Option_1.some(newMinutes), seconds, periodType);
                        this.secondInput = false;
                    }
                    else {
                        var newMinutes = digit;
                        if (digit > 5 && (withPeriodType || withSeconds)) {
                            if (withSeconds) {
                                this.setState({
                                    activeSection: TimeInput_model_1.Section.Seconds,
                                });
                            }
                            else if (withPeriodType) {
                                this.setState({
                                    activeSection: TimeInput_model_1.Section.PeriodType,
                                });
                            }
                            this.secondInput = false;
                        }
                        else {
                            this.secondInput = true;
                        }
                        this.updateTime(hours, Option_1.some(newMinutes), seconds, periodType);
                    }
                }
                break;
            }
            case TimeInput_model_1.Section.Seconds: {
                var newSeconds = void 0;
                if (this.secondInput) {
                    newSeconds = Option_1.isSome(seconds) ? Number("" + seconds.value % 10 + digit) : digit;
                }
                else {
                    newSeconds = digit;
                    this.secondInput = true;
                }
                this.updateTime(hours, minutes, Option_1.some(newSeconds), periodType);
                break;
            }
        }
    };
    RawTimeInput.prototype.step = function (amount) {
        var activeSection = this.state.activeSection;
        var withPeriodType = this.props.withPeriodType;
        var _a = this.props.value, hours = _a.hours, minutes = _a.minutes, seconds = _a.seconds, periodType = _a.periodType;
        switch (activeSection) {
            case TimeInput_model_1.Section.Hours: {
                if (withPeriodType) {
                    this.updateTime(TimeInput_model_1.add(hours, amount, TimeInput_model_1.MAX_VALID_HOURS_FOR_12H_FORMAT), minutes, seconds, periodType);
                }
                else {
                    this.updateTime(TimeInput_model_1.add(hours, amount, TimeInput_model_1.MAX_VALID_HOURS_FOR_24H_FORMAT), minutes, seconds, periodType);
                }
                break;
            }
            case TimeInput_model_1.Section.Minutes: {
                this.updateTime(hours, TimeInput_model_1.add(minutes, amount, TimeInput_model_1.MAX_VALID_MINS_AND_SEC), seconds, periodType);
                break;
            }
            case TimeInput_model_1.Section.Seconds: {
                this.updateTime(hours, minutes, TimeInput_model_1.add(seconds, amount, TimeInput_model_1.MAX_VALID_MINS_AND_SEC), periodType);
                break;
            }
            case TimeInput_model_1.Section.PeriodType: {
                this.updateTime(hours, minutes, seconds, TimeInput_model_1.togglePeriodType(periodType));
                break;
            }
        }
    };
    RawTimeInput.prototype.updateTime = function (newHours, newMinutes, newSeconds, newPeriodType) {
        var _a = this.props, onValueChange = _a.onValueChange, value = _a.value;
        var hours = value.hours, minutes = value.minutes, seconds = value.seconds, periodType = value.periodType;
        TimeInput_model_1.isTimesDifferent({
            hours: newHours,
            minutes: newMinutes,
            seconds: newSeconds,
            periodType: newPeriodType,
        }, {
            hours: hours,
            minutes: minutes,
            seconds: seconds,
            periodType: periodType,
        }) &&
            onValueChange &&
            onValueChange({
                hours: newHours,
                minutes: newMinutes,
                seconds: newSeconds,
                periodType: newPeriodType,
            });
    };
    RawTimeInput.prototype.correctTimeAndUpdate = function () {
        var _a = this.props.value, minutes = _a.minutes, hours = _a.hours, seconds = _a.seconds, periodType = _a.periodType;
        var isMinutesInvalid = pipeable_1.pipe(minutes, Option_1.map(function (min) { return min > TimeInput_model_1.MAX_VALID_MINS_AND_SEC; }));
        var isSecondsInvalid = pipeable_1.pipe(seconds, Option_1.map(function (sec) { return sec > TimeInput_model_1.MAX_VALID_MINS_AND_SEC; }));
        if (Option_1.toNullable(isMinutesInvalid) || Option_1.toNullable(isSecondsInvalid)) {
            var correctedMinutes = Option_1.toNullable(isMinutesInvalid) ? Option_1.some(TimeInput_model_1.MAX_VALID_MINS_AND_SEC) : minutes;
            var correctedSeconds = Option_1.toNullable(isSecondsInvalid) ? Option_1.some(TimeInput_model_1.MAX_VALID_MINS_AND_SEC) : seconds;
            this.updateTime(hours, correctedMinutes, correctedSeconds, periodType);
        }
    };
    RawTimeInput = tslib_1.__decorate([
        pure_1.PURE
    ], RawTimeInput);
    return RawTimeInput;
}(react_1.default.Component));
exports.TimeInput = withTheme_1.withTheme(exports.TIME_INPUT)(with_defaults_1.withDefaults({
    SteppableInput: SteppableInput_1.SteppableInput,
})(RawTimeInput));
//# sourceMappingURL=TimeInput.js.map