"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var layout_constants_1 = require("./layout.constants");
var layout_utils_1 = require("./layout.utils");
var string_1 = require("@devexperts/utils/dist/string/string");
var ActionType;
(function (ActionType) {
    ActionType[ActionType["ItemRemove"] = 0] = "ItemRemove";
    ActionType[ActionType["ItemInsertAt"] = 1] = "ItemInsertAt";
    ActionType[ActionType["ItemInsertNativeAt"] = 2] = "ItemInsertNativeAt";
    ActionType[ActionType["ItemInsertNativeAtEmptyRoot"] = 3] = "ItemInsertNativeAtEmptyRoot";
    ActionType[ActionType["ItemMoveTo"] = 4] = "ItemMoveTo";
    ActionType[ActionType["ItemResize"] = 5] = "ItemResize";
})(ActionType || (ActionType = {}));
exports.remove = function (id) { return ({
    type: ActionType.ItemRemove,
    payload: {
        id: id,
    },
}); };
exports.insertAt = function (item, id, side) { return ({
    type: ActionType.ItemInsertAt,
    payload: {
        item: item,
        id: id,
        side: side,
    },
}); };
exports.insertNativeAt = function (props, id, side) { return ({
    type: ActionType.ItemInsertNativeAt,
    payload: {
        props: props,
        id: id,
        side: side,
    },
}); };
exports.insertNativeAtEmptyRoot = function (props) { return ({
    type: ActionType.ItemInsertNativeAtEmptyRoot,
    payload: {
        props: props,
    },
}); };
exports.moveTo = function (id, destinationId, side) { return ({
    type: ActionType.ItemMoveTo,
    payload: {
        id: id,
        destinationId: destinationId,
        side: side,
    },
}); };
exports.resize = function (id1, size1, id2, size2) { return ({
    type: ActionType.ItemResize,
    payload: {
        id1: id1,
        size1: size1,
        id2: id2,
        size2: size2,
    },
}); };
var createInitial = function () {
    var root_id = string_1.uuid();
    return layout_utils_1.prepareLayout({
        id: root_id,
        size: 1,
        items: [],
        orientation: layout_constants_1.LayoutOrientation.Horizontal,
    });
};
exports.reducer = function (state, action) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    if (state === void 0) { state = createInitial(); }
    switch (action.type) {
        case ActionType.ItemInsertNativeAtEmptyRoot: {
            var props = action.payload.props;
            var root = state.items[state.root_id];
            if (root.type === layout_constants_1.LayoutModelItemType.Layout && layout_utils_1.isEmptyRoot(state, root)) {
                var id = string_1.uuid();
                return tslib_1.__assign(tslib_1.__assign({}, state), { items: tslib_1.__assign(tslib_1.__assign({}, state.items), (_a = {}, _a[root.id] = tslib_1.__assign(tslib_1.__assign({}, root), { item_ids: [id] }), _a[id] = {
                        type: layout_constants_1.LayoutModelItemType.Plain,
                        id: id,
                        size: 1,
                        parent_id: root.id,
                        props: props,
                    }, _a)) });
            }
            return state;
        }
        case ActionType.ItemInsertNativeAt: {
            var _j = action.payload, props = _j.props, id = _j.id, side = _j.side;
            var item = {
                type: layout_constants_1.LayoutModelItemType.Plain,
                size: 0,
                parent_id: '',
                id: string_1.uuid(),
                props: props,
            };
            return exports.reducer(state, exports.insertAt(item, id, side));
        }
        case ActionType.ItemInsertAt: {
            var _k = action.payload, item = _k.item, id = _k.id, side = _k.side;
            var destination_1 = state.items[id];
            if (!destination_1) {
                return state;
            }
            var parent_1 = state.items[destination_1.parent_id];
            if (!parent_1) {
                return state;
            }
            if (destination_1.type === layout_constants_1.LayoutModelItemType.Layout) {
                var orientation_1 = destination_1.orientation;
                if ((orientation_1 === layout_constants_1.LayoutOrientation.Vertical && side === layout_constants_1.ItemSide.Top) ||
                    (orientation_1 === layout_constants_1.LayoutOrientation.Vertical && side === layout_constants_1.ItemSide.Bottom) ||
                    (orientation_1 === layout_constants_1.LayoutOrientation.Horizontal && side === layout_constants_1.ItemSide.Left) ||
                    (orientation_1 === layout_constants_1.LayoutOrientation.Horizontal && side === layout_constants_1.ItemSide.Right)) {
                    var item_ids = side === layout_constants_1.ItemSide.Top || side === layout_constants_1.ItemSide.Left
                        ? tslib_1.__spreadArrays([item.id], destination_1.item_ids) : tslib_1.__spreadArrays(destination_1.item_ids, [item.id]);
                    var items = tslib_1.__assign(tslib_1.__assign({}, state.items), (_b = {}, _b[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { size: 0.5, parent_id: destination_1.id }), _b[destination_1.id] = tslib_1.__assign(tslib_1.__assign({}, destination_1), { item_ids: item_ids }), _b));
                    return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items, destination_1.id) });
                }
                if (parent_1.type === layout_constants_1.LayoutModelItemType.Layout) {
                    var wrapper_1 = layout_utils_1.generateWrapperLayout(item.id, destination_1, parent_1, side);
                    var items = tslib_1.__assign(tslib_1.__assign({}, state.items), (_c = {}, _c[destination_1.id] = tslib_1.__assign(tslib_1.__assign({}, destination_1), { size: 0.5, parent_id: wrapper_1.id }), _c[parent_1.id] = tslib_1.__assign(tslib_1.__assign({}, parent_1), { item_ids: parent_1.item_ids.map(function (itemId) {
                            return itemId === destination_1.id ? wrapper_1.id : itemId;
                        }) }), _c[wrapper_1.id] = wrapper_1, _c[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { size: 0.5, parent_id: wrapper_1.id }), _c));
                    return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items, destination_1.id) });
                }
            }
            if (destination_1.type === layout_constants_1.LayoutModelItemType.Plain) {
                var parent_2 = state.items[destination_1.parent_id];
                if (parent_2.type === layout_constants_1.LayoutModelItemType.Layout) {
                    var orientation_2 = parent_2.orientation;
                    var destinationIndex = parent_2.item_ids.indexOf(id);
                    //check if item can be inserted into existing LayoutModel before destination
                    if ((orientation_2 === layout_constants_1.LayoutOrientation.Horizontal && side === layout_constants_1.ItemSide.Left) ||
                        (orientation_2 === layout_constants_1.LayoutOrientation.Vertical && side === layout_constants_1.ItemSide.Top)) {
                        var size = destination_1.size / 2;
                        var items_1 = tslib_1.__assign(tslib_1.__assign({}, state.items), (_d = {}, _d[parent_2.id] = tslib_1.__assign(tslib_1.__assign({}, parent_2), { item_ids: tslib_1.__spreadArrays(parent_2.item_ids.slice(0, destinationIndex), [
                                item.id
                            ], parent_2.item_ids.slice(destinationIndex)) }), _d[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { size: size, parent_id: parent_2.id }), _d[destination_1.id] = tslib_1.__assign(tslib_1.__assign({}, destination_1), { size: size }), _d));
                        return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items_1, destination_1.id) });
                    }
                    //check if item can be inserted into existing LayoutModel after destination
                    if ((orientation_2 === layout_constants_1.LayoutOrientation.Horizontal && side === layout_constants_1.ItemSide.Right) ||
                        (orientation_2 === layout_constants_1.LayoutOrientation.Vertical && side === layout_constants_1.ItemSide.Bottom)) {
                        var size = destination_1.size / 2;
                        var items_2 = tslib_1.__assign(tslib_1.__assign({}, state.items), (_e = {}, _e[parent_2.id] = tslib_1.__assign(tslib_1.__assign({}, parent_2), { item_ids: tslib_1.__spreadArrays(parent_2.item_ids.slice(0, destinationIndex + 1), [
                                item.id
                            ], parent_2.item_ids.slice(destinationIndex + 1)) }), _e[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { size: size, parent_id: parent_2.id }), _e[destination_1.id] = tslib_1.__assign(tslib_1.__assign({}, destination_1), { size: size }), _e));
                        return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items_2, destination_1.id) });
                    }
                    //wrap destination with nested layout
                    var wrapper_2 = layout_utils_1.generateWrapperLayout(item.id, destination_1, parent_2, side);
                    var items = tslib_1.__assign(tslib_1.__assign({}, state.items), (_f = {}, _f[destination_1.id] = tslib_1.__assign(tslib_1.__assign({}, destination_1), { size: 0.5, parent_id: wrapper_2.id }), _f[parent_2.id] = tslib_1.__assign(tslib_1.__assign({}, parent_2), { item_ids: parent_2.item_ids.map(function (itemId) {
                            return itemId === destination_1.id ? wrapper_2.id : itemId;
                        }) }), _f[wrapper_2.id] = wrapper_2, _f[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { size: 0.5, parent_id: wrapper_2.id }), _f));
                    return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items, parent_2.id) });
                }
            }
            return state;
        }
        case ActionType.ItemMoveTo: {
            var _l = action.payload, id = _l.id, destinationId = _l.destinationId, side = _l.side;
            var item = state.items[id];
            if (item.type === layout_constants_1.LayoutModelItemType.Plain) {
                return exports.reducer(exports.reducer(state, exports.remove(id)), exports.insertAt(item, destinationId, side));
            }
            return state;
        }
        case ActionType.ItemResize: {
            var _m = action.payload, id1 = _m.id1, size1 = _m.size1, id2 = _m.id2, size2 = _m.size2;
            var items = tslib_1.__assign(tslib_1.__assign({}, state.items), (_g = {}, _g[id1] = tslib_1.__assign(tslib_1.__assign({}, state.items[id1]), { size: size1 }), _g[id2] = tslib_1.__assign(tslib_1.__assign({}, state.items[id2]), { size: size2 }), _g));
            return tslib_1.__assign(tslib_1.__assign({}, state), { 
                //it's assumed that both id1 and id2 are in one layout - so invalidate only from one of them
                //also size change does not affect current parent, so invalidation should start from current item
                items: layout_utils_1.invalidateParents(items, id1) });
        }
        case ActionType.ItemRemove: {
            //todo refactor
            var id_1 = action.payload.id;
            var item = state.items[id_1];
            if (item.type === layout_constants_1.LayoutModelItemType.Plain) {
                var parent_3 = state.items[item.parent_id];
                if (parent_3.type === layout_constants_1.LayoutModelItemType.Layout) {
                    var newIds_1 = parent_3.item_ids.filter(function (itemId) { return itemId !== id_1; });
                    //update parent with new ids
                    parent_3 = tslib_1.__assign(tslib_1.__assign({}, parent_3), { item_ids: newIds_1 });
                    var items_3 = tslib_1.__assign(tslib_1.__assign({}, state.items), (_h = {}, _h[parent_3.id] = parent_3, _h));
                    //remove item
                    delete items_3[id_1];
                    //check if last item in parent layout
                    if (newIds_1.length === 1) {
                        //last item - replace parent layout with that item
                        var grandParent = items_3[parent_3.parent_id];
                        if (grandParent) {
                            //not root
                            if (grandParent.type === layout_constants_1.LayoutModelItemType.Layout) {
                                items_3[grandParent.id] = tslib_1.__assign(tslib_1.__assign({}, grandParent), { item_ids: grandParent.item_ids.map(function (itemId) {
                                        return itemId === parent_3.id ? newIds_1[0] : itemId;
                                    }) });
                                //update parent_id to grandParent.id
                                items_3[newIds_1[0]] = tslib_1.__assign(tslib_1.__assign({}, items_3[newIds_1[0]]), { size: parent_3.size, parent_id: grandParent.id });
                                delete items_3[parent_3.id];
                                return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items_3, grandParent.id) });
                            }
                            return state;
                        }
                    }
                    var sizeMap_1 = layout_utils_1.getSizeMap(state, parent_3, 0, 0, 1, 1, 0, 5);
                    var accessor_1 = layout_utils_1.chooseByOrientation(function (size) { return size.width; }, function (size) { return size.height; }, parent_3.orientation);
                    newIds_1.forEach(function (newId, i) {
                        var newItem = state.items[newId];
                        var value = accessor_1(sizeMap_1[newId]);
                        if (newItem.size !== value) {
                            items_3[newItem.id] = tslib_1.__assign(tslib_1.__assign({}, newItem), { size: value });
                        }
                    });
                    return tslib_1.__assign(tslib_1.__assign({}, state), { items: layout_utils_1.invalidateParents(items_3, parent_3.id) });
                }
            }
            return state;
        }
        default: {
            return state || createInitial();
        }
    }
};
//# sourceMappingURL=layout.redux.js.map